@@ -24,8 +24,15 @@
 #
 ###############################################################################
 
-from __future__ import absolute_import, print_function
-
+import asyncio
+from asyncio import iscoroutine
+from asyncio import Future
+try:
+    from types import AsyncGeneratorType
+except ImportError:
+    class AsyncGeneratorType:
+        pass
+import io
 import os
 import sys
 import time
@@ -43,53 +50,6 @@ from txaio._common import _BatchedTimer
 from txaio import _util
 from txaio import _Config
 
-import six
-
-try:
-    import asyncio
-    from asyncio import iscoroutine
-    from asyncio import Future
-
-except ImportError:
-    # Trollius >= 0.3 was renamed
-    # noinspection PyUnresolvedReferences
-    import trollius as asyncio
-    from trollius import iscoroutine
-    from trollius import Future
-
-try:
-    from types import AsyncGeneratorType  # python 3.5+
-except ImportError:
-    class AsyncGeneratorType(object):
-        pass
-
-
-def _create_future_of_loop(loop):
-    return loop.create_future()
-
-
-def _create_future_directly(loop=None):
-    return Future(loop=loop)
-
-
-def _create_task_of_loop(res, loop):
-    return loop.create_task(res)
-
-
-def _create_task_directly(res, loop=None):
-    return asyncio.Task(res, loop=loop)
-
-
-if sys.version_info >= (3, 4, 2):
-    _create_task = _create_task_of_loop
-    if sys.version_info >= (3, 5, 2):
-        _create_future = _create_future_of_loop
-    else:
-        _create_future = _create_future_directly
-else:
-    _create_task = _create_task_directly
-    _create_future = _create_future_directly
-
 
 config = _Config()
 
@@ -238,7 +198,7 @@ class _TxaioFileHandler(logging.Handler, object):
         if isinstance(record.args, dict):
             fmt = record.args.get(
                 'log_format',
-                record.args.get('log_message', u'')
+                record.args.get('log_message', '')
             )
             message = fmt.format(**record.args)
             dt = datetime.fromtimestamp(record.args.get('log_time', 0))
@@ -249,7 +209,7 @@ class _TxaioFileHandler(logging.Handler, object):
                 for line in traceback.format_exception(*record.exc_info):
                     message = message + line
             dt = datetime.fromtimestamp(record.created)
-        msg = u'{0} {1}{2}'.format(
+        msg = '{0} {1}{2}'.format(
             dt.strftime("%Y-%m-%dT%H:%M:%S%z"),
             message,
             os.linesep
@@ -349,22 +309,31 @@ class _AsyncioApi(object):
     using_asyncio = True
 
     def __init__(self, config):
-        if config.loop is None:
-            config.loop = asyncio.get_event_loop()
         self._config = config
 
+    @property
+    def _loop(self):
+        # if configured explicetly, then use this loop
+        if self._config.loop:
+            return self._config.loop
+
+        # otherwise give out the event loop of the thread this is called in
+        # rather fetching the loop once in __init__, which may not neccessarily
+        # be called from the thread we now run the event loop in.
+        return asyncio.get_event_loop()
+
     def failure_message(self, fail):
         """
         :param fail: must be an IFailedFuture
         returns a unicode error-message
         """
         try:
-            return u'{0}: {1}'.format(
+            return '{0}: {1}'.format(
                 fail._value.__class__.__name__,
                 str(fail._value),
             )
         except Exception:
-            return u'Failed to produce failure message for "{0}"'.format(fail)
+            return 'Failed to produce failure message for "{0}"'.format(fail)
 
     def failure_traceback(self, fail):
         """
@@ -379,7 +348,7 @@ class _AsyncioApi(object):
         returns a string
         """
         try:
-            f = six.StringIO()
+            f = io.StringIO()
             traceback.print_exception(
                 fail._type,
                 fail.value,
@@ -388,13 +357,13 @@ class _AsyncioApi(object):
             )
             return f.getvalue()
         except Exception:
-            return u"Failed to format failure traceback for '{0}'".format(fail)
+            return "Failed to format failure traceback for '{0}'".format(fail)
 
     def create_future(self, result=_unspecified, error=_unspecified, canceller=_unspecified):
         if result is not _unspecified and error is not _unspecified:
             raise ValueError("Cannot have both result and error.")
 
-        f = _create_future(loop=self._config.loop)
+        f = self._loop.create_future()
         if result is not _unspecified:
             resolve(f, result)
         elif error is not _unspecified:
@@ -431,7 +400,7 @@ class _AsyncioApi(object):
             if isinstance(res, Future):
                 return res
             elif iscoroutine(res):
-                return _create_task(res, loop=self._config.loop)
+                return self._loop.create_task(res)
             elif isinstance(res, AsyncGeneratorType):
                 raise RuntimeError(
                     "as_future() received an async generator function; does "
@@ -448,7 +417,7 @@ class _AsyncioApi(object):
     def call_later(self, delay, fun, *args, **kwargs):
         # loop.call_later doesn't support kwargs
         real_call = functools.partial(fun, *args, **kwargs)
-        return self._config.loop.call_later(delay, real_call)
+        return self._loop.call_later(delay, real_call)
 
     def make_batched_timer(self, bucket_seconds, chunk_size=100):
         """
@@ -466,7 +435,7 @@ class _AsyncioApi(object):
         """
 
         def get_seconds():
-            return self._config.loop.time()
+            return self._loop.time()
 
         return _BatchedTimer(
             bucket_seconds * 1000.0, chunk_size,
@@ -490,8 +459,11 @@ class _AsyncioApi(object):
                 raise RuntimeError("reject requires an IFailedFuture or Exception")
         future.set_exception(error.value)
 
-    def cancel(self, future):
-        future.cancel()
+    def cancel(self, future, msg=None):
+        if sys.version_info >= (3, 9):
+            future.cancel(msg)
+        else:
+            future.cancel()
 
     def create_failure(self, exception=None):
         """
@@ -515,7 +487,7 @@ class _AsyncioApi(object):
                 res = f.result()
                 if callback:
                     callback(res)
-            except Exception:
+            except (Exception, asyncio.CancelledError):
                 if errback:
                     errback(create_failure())
         return future.add_done_callback(done)
