@@ -24,8 +24,7 @@
 #
 ###############################################################################
 
-from __future__ import absolute_import, division, print_function
-
+import io
 import os
 import sys
 import weakref
@@ -45,17 +44,11 @@ from txaio._iotype import guess_stream_needs_encoding
 from txaio import _Config
 from txaio._common import _BatchedTimer
 from txaio import _util
+from twisted.logger import Logger as _Logger, formatEvent, ILogObserver
+from twisted.logger import globalLogBeginner, formatTime, LogLevel
 
-import six
-
-PY3_CORO = False
-if six.PY3:
-    try:
-        from twisted.internet.defer import ensureDeferred
-        from asyncio import iscoroutinefunction
-        PY3_CORO = True
-    except ImportError:
-        pass
+from twisted.internet.defer import ensureDeferred
+from asyncio import iscoroutinefunction
 
 using_twisted = True
 using_asyncio = False
@@ -77,59 +70,7 @@ _started_logging = False
 _categories = {}
 
 IFailedFuture.register(Failure)
-
-_NEW_LOGGER = False
-try:
-    # Twisted 15+
-    from twisted.logger import Logger as _Logger, formatEvent, ILogObserver
-    from twisted.logger import globalLogBeginner, formatTime, LogLevel
-    ILogger.register(_Logger)
-    _NEW_LOGGER = True
-
-except ImportError:
-    # we still support Twisted 12 and 13, which doesn't have new-logger
-    from zope.interface import Interface
-    from datetime import datetime
-    import time
-
-    # provide our own simple versions of what Twisted new-logger does
-
-    class ILogObserver(Interface):
-        pass
-
-    def formatTime(t):  # noqa
-        dt = datetime.fromtimestamp(t)
-        return six.u(dt.strftime("%Y-%m-%dT%H:%M:%S%z"))
-
-    def formatEvent(event):  # noqa
-        msg = event['log_format']
-        return msg.format(**event)
-
-    class LogLevel:
-        critical = 'critical'
-        error = 'error'
-        warn = 'warn'
-        info = 'info'
-        debug = 'debug'
-        trace = 'trace'
-
-        @classmethod
-        def lookupByName(cls, name):  # noqa
-            return getattr(cls, name)
-
-    class _Logger(ILogger):
-        def __init__(self, **kwargs):
-            self.namespace = kwargs.get('namespace', None)
-
-        def emit(self, level, format='', **kwargs):
-            kwargs['log_time'] = time.time()
-            kwargs['log_level'] = level
-            kwargs['log_format'] = format
-            kwargs['log_namespace'] = self.namespace
-            # NOTE: the other loggers are ignoring any log messages
-            # before start_logging() as well
-            if _observer:
-                _observer(kwargs)
+ILogger.register(_Logger)
 
 
 def _no_op(*args, **kwargs):
@@ -307,7 +248,7 @@ class _LogObserver(object):
         # "Unhandled error in Deferred" -- perhaps this is a Twisted
         # bug?
         if event['log_format'] is None:
-            msg = u'{0} {1}{2}'.format(
+            msg = '{0} {1}{2}'.format(
                 formatTime(event["log_time"]),
                 failure_format_traceback(event['log_failure']),
                 os.linesep,
@@ -319,7 +260,7 @@ class _LogObserver(object):
             # although Logger will already have filtered out unwanted
             # levels, bare Logger instances from Twisted code won't have.
             if 'log_level' in event and self._acceptable_level(event['log_level']):
-                msg = u'{0} {1}{2}'.format(
+                msg = '{0} {1}{2}'.format(
                     formatTime(event["log_time"]),
                     formatEvent(event),
                     os.linesep,
@@ -355,15 +296,10 @@ def start_logging(out=_stdout, level='info'):
     if out:
         _observer = _LogObserver(out)
 
-    if _NEW_LOGGER:
-        _observers = []
-        if _observer:
-            _observers.append(_observer)
-        globalLogBeginner.beginLoggingTo(_observers)
-    else:
-        assert out, "out needs to be given a value if using Twisteds before 15.2"
-        from twisted.python import log
-        log.startLogging(out)
+    _observers = []
+    if _observer:
+        _observers.append(_observer)
+    globalLogBeginner.beginLoggingTo(_observers)
 
 
 _unspecified = object()
@@ -380,7 +316,7 @@ class _TxApi(object):
         returns a unicode error-message
         """
         try:
-            return u'{0}: {1}'.format(
+            return '{0}: {1}'.format(
                 fail.value.__class__.__name__,
                 fail.getErrorMessage(),
             )
@@ -400,11 +336,11 @@ class _TxApi(object):
         returns a string
         """
         try:
-            f = six.StringIO()
+            f = io.StringIO()
             fail.printTraceback(file=f)
             return f.getvalue()
         except Exception:
-            return u"Failed to format failure traceback for '{0}'".format(fail)
+            return "Failed to format failure traceback for '{0}'".format(fail)
 
     def create_future(self, result=_unspecified, error=_unspecified, canceller=None):
         if result is not _unspecified and error is not _unspecified:
@@ -425,8 +361,11 @@ class _TxApi(object):
 
     def as_future(self, fun, *args, **kwargs):
         # Twisted doesn't automagically deal with coroutines on Py3
-        if PY3_CORO and iscoroutinefunction(fun):
-            return ensureDeferred(fun(*args, **kwargs))
+        if iscoroutinefunction(fun):
+            try:
+                return ensureDeferred(fun(*args, **kwargs))
+            except TypeError as e:
+                return create_future_error(e)
         return maybeDeferred(fun, *args, **kwargs)
 
     def is_future(self, obj):
@@ -478,7 +417,7 @@ class _TxApi(object):
                 raise RuntimeError("reject requires a Failure or Exception")
         future.errback(error)
 
-    def cancel(self, future):
+    def cancel(self, future, msg=None):
         future.cancel()
 
     def create_failure(self, exception=None):
